package space.kklochko.spring_rest_example.controllers;

import jakarta.persistence.EntityManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import space.kklochko.spring_rest_example.db.repositories.DepartmentRepository;
import space.kklochko.spring_rest_example.models.Department;
import space.kklochko.spring_rest_example.models.rest.RestError;
import space.kklochko.spring_rest_example.models.rest.RestMessage;

import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/api/v1/departments")
public class DepartmentController {
    @Autowired
    EntityManager manager;

    @Autowired
    DepartmentRepository departmentRepository;

    @PostMapping
    @ResponseBody
    public ResponseEntity<Object> create(@RequestBody Department department) {
        Department isExist = departmentRepository.read(department.getId());

        if(isExist != null)
            return new ResponseEntity<>(new RestError("Failed to create the department, because the id is already used!!!"), HttpStatus.CONFLICT);

        boolean successful = departmentRepository.create(department);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to create the department."), HttpStatus.INTERNAL_SERVER_ERROR);

        return new ResponseEntity<>(new RestMessage("The department created successfully."), HttpStatus.CREATED);
    }

    @GetMapping("/{id}")
    @ResponseBody
    public ResponseEntity<?> get(@PathVariable UUID id) {
        Department department = departmentRepository.read(id);

        if(department == null)
            return new ResponseEntity<>(new RestError("The department not found."), HttpStatus.NOT_FOUND);

        return new ResponseEntity<>(department, HttpStatus.OK);
    }

    @GetMapping
    @ResponseBody
    public List<Department> getAll() {
        return departmentRepository.readAll();
    }

    @PutMapping
    @ResponseBody
    public ResponseEntity<Object> update(@RequestBody Department department) {
        Department isExist = departmentRepository.read(department.getId());

        if(isExist == null)
            return new ResponseEntity<>(new RestError("Failed to update the department, because the department does not exist!!!"), HttpStatus.NOT_FOUND);

        boolean successful = departmentRepository.update(department);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to update the department."), HttpStatus.INTERNAL_SERVER_ERROR);

        return new ResponseEntity<>(new RestMessage("The department updated successfully."), HttpStatus.OK);
    }

    @DeleteMapping("/{id}")
    @ResponseBody
    public ResponseEntity<Object> delete(@PathVariable UUID id) {
        boolean successful = departmentRepository.delete(id);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to delete the department, because the department does not exist!!!"), HttpStatus.NOT_FOUND);

        return new ResponseEntity<>(new RestMessage("The department deleted successfully."), HttpStatus.OK);
    }
}

