package space.kklochko.spring_rest_example.controllers;

import jakarta.persistence.EntityManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import space.kklochko.spring_rest_example.db.repositories.IndicatorRepository;
import space.kklochko.spring_rest_example.models.Indicator;
import space.kklochko.spring_rest_example.models.rest.RestError;
import space.kklochko.spring_rest_example.models.rest.RestMessage;

import java.util.List;
import java.util.UUID;

@RestController
@RequestMapping("/api/v1/indicators")
public class IndicatorController {
    @Autowired
    EntityManager manager;

    @Autowired
    IndicatorRepository indicatorRepository;

    @PostMapping
    @ResponseBody
    public ResponseEntity<Object> create(@RequestBody Indicator indicator) {
        Indicator isExist = indicatorRepository.read(indicator.getId());

        if(isExist != null)
            return new ResponseEntity<>(new RestError("Failed to create the indicator, because the id is already used!!!"), HttpStatus.CONFLICT);

        boolean successful = indicatorRepository.create(indicator);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to create the indicator."), HttpStatus.INTERNAL_SERVER_ERROR);

        return new ResponseEntity<>(new RestMessage("The indicator created successfully."), HttpStatus.CREATED);
    }

    @GetMapping("/{id}")
    @ResponseBody
    public ResponseEntity<?> get(@PathVariable UUID id) {
        Indicator indicator = indicatorRepository.read(id);

        if(indicator == null)
            return new ResponseEntity<>(new RestError("The indicator not found."), HttpStatus.NOT_FOUND);

        return new ResponseEntity<>(indicator, HttpStatus.OK);
    }

    @GetMapping
    @ResponseBody
    public List<Indicator> getAll() {
        return indicatorRepository.readAll();
    }

    @PutMapping
    @ResponseBody
    public ResponseEntity<Object> update(@RequestBody Indicator indicator) {
        Indicator isExist = indicatorRepository.read(indicator.getId());

        if(isExist == null)
            return new ResponseEntity<>(new RestError("Failed to update the indicator, because the indicator does not exist!!!"), HttpStatus.NOT_FOUND);

        boolean successful = indicatorRepository.update(indicator);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to update the indicator."), HttpStatus.INTERNAL_SERVER_ERROR);

        return new ResponseEntity<>(new RestMessage("The indicator updated successfully."), HttpStatus.OK);
    }

    @DeleteMapping("/{id}")
    @ResponseBody
    public ResponseEntity<Object> delete(@PathVariable UUID id) {
        boolean successful = indicatorRepository.delete(id);

        if(!successful)
            return new ResponseEntity<>(new RestError("Failed to delete the indicator, because the indicator does not exist!!!"), HttpStatus.NOT_FOUND);

        return new ResponseEntity<>(new RestMessage("The indicator deleted successfully."), HttpStatus.OK);
    }
}

